class save_formats:
	'''
	For representing the save format for scraped data 
	The attributes just act as quick to access identifiers for the save type when only the save type is needed


	''' 

	EXCEL_FILE = "EXCEL_SPREADSHEET_FORMAT"
	CSV_FILE = "CSV_FORMAT"
	JSON  = "jSON OBJECT"
	JSON_FILE = "JSON FILE " 
	TEXT_FILE= "TEXT BASED FILE"
	IMAGE = "PNGs, JPEGS ....."
